/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.scheduler.multiple.internal;

import com.liferay.portal.kernel.cluster.ClusterInvokeAcceptor;
import com.liferay.portal.kernel.cluster.ClusterInvokeThreadLocal;
import com.liferay.portal.kernel.scheduler.SchedulerEngine;
import com.liferay.portal.kernel.scheduler.StorageType;

import java.io.Serializable;

import java.util.Map;

import org.osgi.service.component.annotations.Component;

/**
 * @author Michael C. Han
 */
@Component(immediate = true, service = ClusterInvokeAcceptor.class)
public class SchedulerClusterInvokeAcceptor implements ClusterInvokeAcceptor {

	@Override
	public boolean accept(Map<String, Serializable> context) {
		if (ClusterInvokeThreadLocal.isEnabled()) {
			return false;
		}

		boolean pluginReady = (Boolean)context.get(
			ClusterSchedulerEngine.PLUGIN_READY);

		if (!pluginReady) {
			return false;
		}

		boolean portalReady = (Boolean)context.get(
			ClusterSchedulerEngine.PORTAL_READY);

		if (!portalReady) {
			return false;
		}

		Boolean schedulerClusterInvoking = (Boolean)context.get(
			SchedulerEngine.SCHEDULER_CLUSTER_INVOKING);

		if ((schedulerClusterInvoking != null) && !schedulerClusterInvoking) {
			return false;
		}

		StorageType storageType = (StorageType)context.get(
			SchedulerEngine.STORAGE_TYPE);

		if (storageType == StorageType.PERSISTED) {
			return false;
		}

		return true;
	}

}